/*
 * DOGM_SPI.c
 * SPI - Software-Schnittstelle fr DOGM162L-A Display
 *  Created on: 22.05.2014
 *      Author: DL9MWE
 *      unter Verwendung von Code von DL9GFA und Peter Denneger  tnx!

 Anschluss des LC-Displays DOGM162L-A:
 Pins 25,26,30,31,32,33,34,35,36,36,37,40 an +3,3V
 Pins 23 und 27 an GND
 (Folien-)Kondensator 1uF an Pins 21 und 22
 (Folien-)Kondensator 1uF an Pins 24 und 25
  Pin 28 SI   (MSP430-Portpin: DOGM162_SI)
  Pin 29 CLK  (MSP430-Portpin: DOGM162_CLK)
  Pin 38 CSB  (MSP430-Portpin: DOGM162_CSB)
  Pin 39 RS   (MSP430-Portpin: DOGM162_RS)
 */
#include "main.h"

// ---------- interne Funktionen ----------
static void lcd_byte( unsigned char cc );
static void lcd_command(unsigned char cc );
static unsigned char put_uidigit(unsigned int *uiX, unsigned char stelle, unsigned char print);


//				------- extern verfgbare Funktionen -------
// Initialiserung des DOGM162,  2 Zeilen mit je 16 Zeichen via 4-wire-SPI
void vLCD_init(void) {
	// SPI 4-wire Outputs
	PORTDOGM_DIR |= DOGM162_RS | DOGM162_CSB | DOGM162_CLK | DOGM162_SI;
	PORTDOGM_SEL = 0;
	PORTDOGM_SEL2 = 0;		// normal IO-Function
	PORTDOGM_OUT |= DOGM162_RS | DOGM162_CSB | DOGM162_CLK | DOGM162_SI;	// alles auf 1

	_delay_ms( 15 );			// wait 15ms
	lcd_command( 0x39 );	// Function Set: 8 Bit, 2 lines, Instruction table 1
	lcd_command( 0x14 );	// 1/5 lcd_command( 0x1C );	// Bias Set: 1/4
	lcd_command( 0x55 );	// lcd_command( 0x52 );	// Power control
	lcd_command( 0x6D );	// lcd_command( 0x69 );	// Follower Control: On,
	lcd_command( 0x78 );	// lcd_command( 0x74 ); // Contrast Set
	lcd_command( 0x38 );	// Function Set: Instruction table 0
	lcd_command( 0x0F );	// Display On, Cursor On, Cursor Blink
	lcd_command( 0x01 );	// delete Display, Cursor Home
	lcd_command( 0x06 );	// Cursor auto-increment
	lcd_command( 0x0C );	// Display on, no cursor, no blink
}

// ein einzelnes Zeichen schreiben
void vLCD_putc(unsigned char cc) {
	PORTDOGM_OUT |= DOGM162_RS;	//LCD_RS = 1, i.e. Ziel ist Data Register
	lcd_byte( cc );
}

// String schreiben (durch 0\x00 beendet)
void vLCD_puts(char* ss) {
	while(*ss)
		vLCD_putc(*ss++);
}

//********************************************************************************
// Description  : write an unsigned integer to LCD
// Input        : unsigned integer uiX
// Output       : none
//		modified by DL9MWE (62 Bytes krzer als das Original)
//********************************************************************************
// vorzeichenlose 16-Bit-Zahl als Dezimal-Wert auf dem Display ausgeben
void vLCD_putui(unsigned int uiX) {
	unsigned char stelle, print = 0;

	for (stelle = 4; stelle; --stelle) {
		print = put_uidigit(&uiX, stelle, print); // fhrende Nullen nicht ausgeben
	}
	vLCD_putc(uiX + 0x30); // die Einerstelle immer ausgeben
}
// Hilfsfunktion: gibt eine Stelle auf dem LCD aus (fhrende Nullen werden ignoriert)
unsigned char put_uidigit(unsigned int *uiX, unsigned char stelle,
		unsigned char print) {
	unsigned int y, dezimal = 1;
	while (stelle) {
		dezimal = dezimal * 10;
		--stelle;
	}
	y = *uiX / dezimal;
	if (y || print) {
		vLCD_putc(y + 0x30);
		print = 1;
		*uiX -= y * dezimal;
	}
	return print;
}


//********************************************************************************
// Description  : write an signed integer to LCD
// Input        : signed integer iX
// Output       : none
//********************************************************************************
void vLCD_puti(int iX) {
	if (iX < 0) {
		vLCD_putc('-');
		iX = ~iX + 1;
	}
	vLCD_putui((unsigned int) iX);
}


// Display lschen, Cursor auf 1,1
void vLCD_clear(void) {
	lcd_command( 0x01 );
	_delay_ms(2);		// min 1,08 ms
}


//********************************************************************************
// Description  : set the LCD display position x=1..16 y=1..2
//              :
// Input        : ucX - X-position
//              : ucY - Y-position
// Output       : none
//********************************************************************************
void vLCD_gotoxy(unsigned char ucX, unsigned char ucY) {
  unsigned char cc;
	cc = 0x80 | --ucX;	// Befehl (0x80) fr Zeichenposition (ucX)
	if(--ucY)
		cc |= 0x40;		// Zeile 2 ist gemeint
	lcd_command(cc);
}


// ---------- interne Funktionen ----------
static void lcd_byte( unsigned char cc )	// Ausgabe eines Bytes an LCD via SPI
{
	unsigned char ii;

	PORTDOGM_OUT &= ~DOGM162_CSB;			//LCD_CS = 0;
	for( ii = 8; ii; ii-- ) {
		_delay_10us( 1 );
		PORTDOGM_OUT &= ~DOGM162_CLK;		//LCD_CLK = 0;
		if( cc & 0x80 )
			PORTDOGM_OUT |= DOGM162_SI;		//LCD_DI = 1;
		else
			PORTDOGM_OUT &= ~DOGM162_SI;	//LCD_DI = 0;
		cc <<= 1;
		_delay_10us( 1 );
		PORTDOGM_OUT |= DOGM162_CLK;		//LCD_CLK = 1;
	}
	PORTDOGM_OUT |= DOGM162_CSB;			//LCD_CS  = 1;
	_delay_10us( 3 );						// 30us
}


static void lcd_command(unsigned char cc )	// Ausgabe eines Kommandos an LCD
{
	PORTDOGM_OUT &= ~DOGM162_RS;	//LCD_RS = 0, i.e. Ziel ist Instruction Register
	lcd_byte( cc );
	_delay_ms( 2 );					// wait 2ms
	PORTDOGM_OUT |= DOGM162_RS;		//RS = 1
}


// Ende DOGM_SPI.c

